@@ -8,7 +8,12 @@ __all__ = ['TestSession']
 
 
 class TestSession(SessionInterface):
-    memory_instance = DynamoDbSessionInstance()
+    class TestInstance(DynamoDbSessionInstance):
+        def abandon(self):
+            """ Clears the session but doesn't do any DynamoDB actions """
+            self.clear()
+
+    memory_instance: DynamoDbSessionInstance = TestInstance()
 
     def open_session(self, app: Flask, request: Request) -> DynamoDbSessionInstance:
         return self.memory_instance
